-------------------------------------------------------------------------------- 
-- UPC - EETAC - EEL - CSD - DIGSYS       
--------------------------------------------------------------------------------
-- Light control prototype in LAB6
-- CLK_Generator chip
-- Strategy: Schematic using frequency dividers and TFF to square signals
---------------------------------------------------
-- Project P8. Lecture L8.2 on designing CLK_generators  
-- https://digsys.upc.edu/csd/P08/L08_2/L8_2.html

-- Translation Fig. 19 schematic in LAB6 at: 
-- https://digsys.upc.edu/csd/P06/lab06/lab6.html 
--------------------------------------------------------------------------------
-- OSC_CLK_in = 50 MHz  from the DE10-Lite oscillator
-- CLK_200Hz_SQ to run the FSM
--------------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
	
ENTITY CLK_Generator IS  
      PORT(	
			OSC_CLK_in		:	IN		STD_LOGIC;
			CD 				:	IN		STD_LOGIC;
			CE				:	IN		STD_LOGIC;
			CLK_200Hz_SQ	:	OUT	STD_LOGIC
		);
END CLK_Generator;

ARCHITECTURE schematic OF CLK_Generator  IS  
-- Components
	COMPONENT Freq_Div_125000	IS                 
		PORT(	
			CD,CLK,CE		: 	IN  std_logic;
			TC125000		:	OUT std_logic
		);
	END COMPONENT;	
	
	COMPONENT T_FF IS
		Port (	
			CLK 	: IN	STD_LOGIC;
			CD 		: IN	STD_LOGIC;
			T 		: IN	STD_LOGIC;
			Q 		: OUT	STD_LOGIC
		);
	END COMPONENT;					

-- Signals for connecting components together (just the internal wires)
	SIGNAL CLK				:	std_logic;
	SIGNAL Pulsed_400Hz		:	std_logic;

		
	BEGIN  
-- Instantiation of components

	Chip1	: 	Freq_Div_125000
	PORT MAP (	   	
-- from component name 	=> to signal or port name
			CLK			=> CLK,
			CD 			=> CD,
			CE 			=> CE,
			TC125000	=> Pulsed_400Hz					  
		);
	
	Chip2	: T_FF
		PORT MAP (
	-- from component name 	=> to signal or port name	
			CLK		=> CLK,
			CD 		=> CD,
			T		=> Pulsed_400Hz,
			Q 		=> CLK_200Hz_SQ				  
		);		
	
-- Connections and logic between components

	-- The internal CLK form the external quartz crystal 
	CLK <= OSC_CLK_in;
	
END schematic ;
